

#include "ui/support/x_motif/fntcache.h"
#include "ui/applic.h"
#include <iostream.h>

FontCache::FontCache ()
    : _time (0)
{
}

XmFontList FontCache::FindFont (UI_Font* fnt)
{
    _time++;
    CL_String fontName = fnt->NativeName();
    CacheEntry* p = (CacheEntry*) _cache[fontName];
    if (!p) {
        p = new CacheEntry;
        p->refCount = 0;
        p->timeStamp = _time;
        Display* dpy = YACLApp()->AppDisplay();
        UI_ResourceHandle h = fnt->Handle();
        p->ptr = XmFontListCreate (XQueryFont (dpy, h),
                                   XmSTRING_DEFAULT_CHARSET);
        _cache.Add (fontName, (long) p);
        if (_cache.Size() >= MaxCacheSize)
            Sweep ();
    }
    else {
        p->timeStamp = _time;
        // Update the time stamp on the font that was accessed
    }
    p->refCount++;
    return p->ptr;
}

CL_String FontCache::Sweep  ()
{
    // Find the least recently accessed font...
    CL_StringIntMapIterator itr (_cache);
    CL_IntStringAssoc oldestAssoc;
    long oldestTime =  0x7fffffff;
    CL_String oldestFont;
    CacheEntry* p;
    while (itr.More ()) {
        CL_StringIntAssoc assoc = itr.Next ();
        p = (CacheEntry*) assoc.value;
        if (p && p->timeStamp < oldestTime) {
            oldestTime = p->timeStamp;
            oldestFont = assoc.key;
        }
    }

    // ... and remove it from the cache
    p = (CacheEntry*) _cache[oldestFont];
    if (p) {
        XmFontListFree (p->ptr);
        delete p;
        _cache.Remove (oldestFont);
        return oldestFont;
    }
    return "";
}

FontCache::~FontCache ()
{
}
