

#include <X11/Xlib.h>

#include "ui/applic.h"
#include "ui/composit.h"
#include "ui/dsplsurf.h"
#include "ui/dsinmem.h"
#include "ui/brush.h"
#include "ui/pen.h"
#include "ui/shadorec.h"
#include "ui/support/x_yacl/window.h"
#include <iostream.h>

YVE_Window::YVE_Window (UI_VisualObject* vObj, 
                        const UI_Rectangle& shape, Window pw ) 
: _vObj (vObj)
{
    _Init (shape, pw);
}

YVE_Window::YVE_Window (UI_VisualObject* vObj, Window pw ) 
: _vObj (vObj)
{
    UI_Rectangle shape = vObj ? vObj->Shape() : UI_Rectangle (0, 0, 0, 0);
    _Init (shape, pw);
}



void YVE_Window::_Init (const UI_Rectangle& shape, Window pw)
{
    Display* dp = YACLApp()->AppDisplay();
    int screenNo = DefaultScreen (dp);
    Window parentW = pw ? pw : (_vObj && _vObj->Parent()
                                ? _vObj->Parent()->ViewHandle()
                                : RootWindow (dp, screenNo));
    _window = XCreateSimpleWindow
        (dp, parentW, shape.Left(), shape.Top(),
         shape.Width(), shape.Height(), 0,
         BlackPixel (dp, screenNo), _vObj->Background().XPixel()); 

    XSetWindowAttributes attr;
    attr.override_redirect = True ;
    ulong valuemask = 0 ;
    valuemask |= CWOverrideRedirect ;
    XChangeWindowAttributes(dp, _window, valuemask, &attr) ;
    XSelectInput (dp, _window,
                  EnterWindowMask | LeaveWindowMask |
                  PointerMotionMask |
                  Button1MotionMask | Button2MotionMask | Button3MotionMask |
                  ExposureMask |
                  KeyPressMask | KeyReleaseMask |
                  ButtonPressMask | ButtonReleaseMask | StructureNotifyMask
                  | FocusChangeMask);
 
}


YVE_Window::~YVE_Window ()
{
    XDestroyWindow (YACLApp()->AppDisplay(), _window);
}

void YVE_Window::Reshape (const UI_Rectangle& )
{
}


YVE_StringWindow::YVE_StringWindow (UI_VisualObject* vObj)
: YVE_Window (vObj)
{
}

const short SHADOW_THICKNESS = 2;

// static Pixmap _maskPixmap;
// static unsigned char gray50_bits[] = {
//    0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa,
//    0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa,
//    0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa};

// void _BeginStipple (UI_DrawingSurface& sfc)
// {
//     Display* d = YACLApp()->AppDisplay();
//     _maskPixmap = XCreateBitmapFromData
//         (d, sfc._Drawable(), (char*) gray50_bits, 16, 16);
//     XGCValues val;
//     val.stipple    = _maskPixmap;
//     val.fill_style = FillStippled;
//     val.function   = GXandReverse;
//     XChangeGC (d, sfc.Handle(), GCFunction |GCFillStyle|GCStipple, &val);
//     XSetStipple (d, sfc.Handle(), _maskPixmap);
// }



// void _EndStipple (UI_DrawingSurface& sfc)
// {
//     if (_maskPixmap) {
//         XGCValues val;
//         Display* d = YACLApp()->AppDisplay();
//         val.stipple = _maskPixmap;
//         val.fill_style = FillStippled;
//         val.function   = GXinvert;
//         XChangeGC (d, sfc.Handle(), GCFunction | GCFillStyle | GCStipple,
//         &val);
//         XFreePixmap (d, _maskPixmap);
//         _maskPixmap = 0;
//     }
// }

void YVE_StringWindow::Draw (const char* content, UI_ReliefStyle style,
                             UI_TextStyle tStyle)
{
    UI_DisplaySurface& sfc = _vObj->CreateDisplaySurface ();

    _content = content;
    UI_Rectangle& shape = _vObj->Shape();
    long w = shape.Width(), h = shape.Height();
    UI_DwgSurfaceInMemory memDS (w, h, &sfc);
    _Rebuild (w, h, memDS, style, tStyle);
    memDS.CopyTo (sfc);
    _vObj->DestroyDisplaySurface();
}


void YVE_StringWindow::_Rebuild (long w, long h,
                                 UI_DrawingSurface& memDS,
                                 UI_ReliefStyle style,
                                 UI_TextStyle tStyle)
{
    short xOffs = 0, yOffs = 0;
    memDS.ColorRectangle (UI_Rectangle (0, 0, w, h), 
                          _vObj->Background());

    switch (style) {
    case UIRelief_Raised: {
        UI_ShadowRectangle r (0, 0, w, h, UI_ShadowRectangle::Raised,
                              SHADOW_THICKNESS);
        r.DrawOn (memDS);
        xOffs = 2 * SHADOW_THICKNESS;
        yOffs = 2 * SHADOW_THICKNESS + 2;
        break;
    }
        
    case UIRelief_Recessed: {
        UI_ShadowRectangle r (0, 0, w, h, 
                              UI_ShadowRectangle::Recessed, SHADOW_THICKNESS);
        r.DrawOn (memDS);
        xOffs = 2 * SHADOW_THICKNESS + 4; // Note that these constants are
                                          // larger, by 4, than the ones in
                                          // the Raised case, because we
                                          // want the recessed effect.
        yOffs = 2 * SHADOW_THICKNESS + 6;
        break;
    }
        
    case UIRelief_Plain:
        if (_vObj->BorderShown()) {
            UI_Rectangle r (0, 0, w-1, h-1);
            r.DrawOn (memDS);
        }
        break;
    };

    if (_content.Size()) {
        UI_Rectangle textRect (xOffs, yOffs, w  - 2*xOffs,
                               h - 2*yOffs);
        UI_Color penColor = _vObj->IsEnabled() ? _vObj->Foreground()
            : _vObj->DisabledForeground();
        memDS.Pen() = UI_PenDescriptor (1, penColor, UIPen_Solid);
        memDS.WriteString (_content.AsPtr(), textRect, tStyle);

    }
}


